@@ -199,7 +199,7 @@ FE.out
 	
 	
 	
-	effects.splm<-function(object,...){
+effects.splm<-function(object,...){
 	x<-object
 	if (class(x) != "splm") stop(paste("methos not implemented for objects of class", class(x)))
 	if (class(x) != "splm" && (x$type != "fixed effects lag" || x$type != "fixed effects error")) stop(paste("methos not implemented for objects of type", x$type))
@@ -313,98 +313,3 @@ out<-rbind(object$INTTable,object$SETable,object$TETable)
 }
 	}
 
-
-write.effects.splm <-
-function (x, file = "effects", ncolumns = if (is.character(x)) 1 else 5, 
-    append = FALSE, sep = ","){
-    	object<-x
-	effects<-object$effects
-if(effects=="tpfe"){
-out<-rbind(object$INTTable,object$TETable)
-write(out, file=file, ncolumns=ncolumns, append=append, sep=sep)
-
-}
-	
-if(effects=="spfe"){
-out<-rbind(object$INTTable,object$SETable)
-write(out, file=file, ncolumns=ncolumns, append=append, sep=sep)
-}
-if(effects=="sptpfe"){
-out<-rbind(object$INTTable,object$SETable,object$TETable)
-write(out, file=file, ncolumns=ncolumns, append=append, sep=sep)
-}
-	}
-
-
-
-
-`print.effects.splm` <-
-function(x,digits= max(3, getOption("digits") - 2),
-...){
-	object<-x
-	effects<-object$effects
-if(effects=="tpfe"){
-	  cat("\nIntercept:\n")
-  printCoefmat(object$INTTable,digits=digits, signif.legend=FALSE)
-
- cat("\n")  
-
-	  cat("\nTime period fixed effects:\n")
-  printCoefmat(object$TETable,digits=digits)
-
-out<-rbind(object$INTTable,object$TETable)
-}
-	
-if(effects=="spfe"){
-	  cat("\nIntercept:\n")
-  printCoefmat(object$INTTable,digits=digits,signif.legend=FALSE)
-
- cat("\n")  
- 
-	  cat("\nSpatial fixed effects:\n")
-  printCoefmat(object$SETable,digits=digits)
-
-
-out<-rbind(object$INTTable,object$SETable)
-}
-if(effects=="sptpfe"){
-	  cat("\nIntercept:\n")
-  printCoefmat(object$INTTable,digits=digits,signif.legend=FALSE)
-
- cat("\n")  
- 
-	  cat("\nSpatial fixed effects:\n")
-  printCoefmat(object$SETable,digits=digits,signif.legend=FALSE)
- 
-  cat("\n")  
-   
-  	  cat("\nTime period fixed effects:\n")
-  printCoefmat(object$TETable,digits=digits)
-
-out<-rbind(object$INTTable,object$SETable,object$TETable)
-}
-	}
-
-
-
-
-`write.effects.splm` <-
-function (x, file = "effects", ncolumns = if (is.character(x)) 1 else 5, 
-    append = FALSE, sep = ","){
-    	object<-x
-	effects<-object$effects
-if(effects=="tpfe"){
-out<-rbind(object$INTTable,object$TETable)
-write(out, file=file, ncolumns=ncolumns, append=append, sep=sep)
-
-}
-	
-if(effects=="spfe"){
-out<-rbind(object$INTTable,object$SETable)
-write(out, file=file, ncolumns=ncolumns, append=append, sep=sep)
-}
-if(effects=="sptpfe"){
-out<-rbind(object$INTTable,object$SETable,object$TETable)
-write(out, file=file, ncolumns=ncolumns, append=append, sep=sep)
-}
-	}
