@@ -11,21 +11,36 @@ spml <- function(formula, data, index=NULL, listw, listw2=listw, na.action,
 
   ## check class(listw)
   checklw <- function(x) {
+# if(model == "within"){
+	
+	# if("matrix" %in% class(x)) x <- Matrix(x)
+		# if("listw" %in% class(x)) x <- listw2dgCMatrix(x)
+			# if("Matrix" %in% class(x)) x <- x
+	
+			  # else stop("'listw' has to be 'listw', 'matrix', or 'Matrix' when model is within")
+	
+			# }  	
+# else{
+    
     if(!("listw" %in% class(x))) {
       if("matrix" %in% class(x)) {
-        require(spdep)
+        #require(spdep)
         x <- listw2mat(x)
-      } else {
+      } 
+      else {
         stop("'listw' has to be either a 'listw' or a 'matrix' object")
       }}
+      # }
     return(x)
   }
+
   checklw(listw)
   checklw(listw2)
 
   ## dimensions check is moved downstream
 
   switch(match.arg(model), within={
+  
     if(lag) {
       model <- switch(match.arg(spatial.error), b="sarar",
                       kkp="sarar", none="lag")
@@ -37,6 +52,7 @@ spml <- function(formula, data, index=NULL, listw, listw2=listw, na.action,
     }
     effects <- switch(match.arg(effect), individual="spfe",
                       time="tpfe", twoways="sptpfe")
+
     res <- spfeml(formula=formula, data=data, index=index,
                   listw=listw, listw2=listw2, na.action,
                   model=model, effects=effects,
